-- Controls the size of each search box. The larger this is the slower the search phase which can break everything.
SEARCH_BOX_SCALE = 1/10


-- Helper function used in an objects constructor to setup its metatable correctly allowing for basic inheritence.
function newObject(class, o, read_only)
    o = o or {}
    setmetatable(o, class)
    class.__index = class
    if read_only then
        makeReadOnly(o)
    end
    return o
end

function makeReadOnly(table)
    local mt = getmetatable(table)
    if not mt then
        mt = {}
        if not table then print(debug.traceback()) end
        setmetatable(table,mt)
    end
    mt.__newindex = function(t,k,v)
        error("Attempt to update a read-only table", 2)
    end
    return table
end

-- Simple immutable vector class
Vector={}
function Vector:new(o)
    o.x = o.x or o[1]
    o.y = o.y or o[2]
    return newObject(self, o, true)
end

function Vector:__add(vector)
    local x,y = Vector.getXY(vector)
    return Vector:new{self.x + x, self.y + y}
end

function Vector:__sub(vector)
    local x,y = Vector.getXY(vector)
    return Vector:new{self.x - x, self.y - y}
end

function Vector:__div(divisor)
    return Vector:new{self.x / divisor, self.y / divisor}
end

function Vector:__mul(multiplicand)
    return Vector:new{self.x * multiplicand, self.y * multiplicand}
end

function Vector.getXY(vector)
    return vector.x or vector[1], vector.y or vector[2]
end

function Vector:length()
    return math.sqrt(self.x^2 + self.y^2)
end

function Vector:normalize()
    return self / self:length()
end

function Vector:__tostring()
    return "(" .. self.x .. ", " .. self.y .. ")"
end


PlantLocation={}
function PlantLocation:new(o)
    if o.num_move_steps then
        o.move_btn = PLANT_LOCATIONS[o.direction_vector].move_btn
        o.direction_vector = o.direction_vector * o.num_move_steps
    else
        o.num_move_steps = 1
    end
    PLANT_LOCATIONS[o.direction_vector] = o
    PLANT_LOCATIONS[PLANT_LOCATIONS.next] = o
    PLANT_LOCATIONS.next = PLANT_LOCATIONS.next + 1
    -- o.box = makeSearchBox(o.direction_vector)
    return newObject(PlantLocation, o, true)
end

function PlantLocation:move()
    for step=1,self.num_move_steps do
        click(self.move_btn)
    end
end

function setupPlantLocations()
    NORTH   = Vector:new{0,-1}
    SOUTH   = Vector:new{0,1}
    WEST    = Vector:new{-1,0}
    EAST    = Vector:new{1,0}
    NORTH_WEST = NORTH + WEST
    NORTH_EAST = NORTH + EAST
    SOUTH_WEST = SOUTH + WEST
    SOUTH_EAST = SOUTH + EAST
    DOUBLE_SOUTH = SOUTH * 2
    DOUBLE_NORTH = NORTH * 2
    DOUBLE_WEST = WEST * 2
    DOUBLE_EAST = EAST * 2

    MOVE_BTNS = {}
    PLANT_LOCATIONS = {next=1}
    PlantLocation:new{direction_vector=NORTH, move_btn=Vector:new{59, 51}}
    PlantLocation:new{direction_vector=EAST ,move_btn=Vector:new{84, 74}}
    PlantLocation:new{direction_vector=SOUTH ,move_btn=Vector:new{60, 98}}
    PlantLocation:new{direction_vector=WEST ,move_btn=Vector:new{37, 75}}
    PlantLocation:new{direction_vector=NORTH, num_move_steps=2}
    PlantLocation:new{direction_vector=WEST, num_move_steps=2}
    PlantLocation:new{direction_vector=EAST, num_move_steps=2}
    PlantLocation:new{direction_vector=SOUTH, num_move_steps=2}
    PlantLocation:new{direction_vector=NORTH_EAST, move_btn=Vector:new{75, 62}}
    PlantLocation:new{direction_vector=NORTH_WEST, move_btn=Vector:new{45,60}}
    PlantLocation:new{direction_vector=SOUTH_EAST, move_btn=Vector:new{75, 91}}
    PlantLocation:new{direction_vector=SOUTH_WEST, move_btn=Vector:new{45, 87}}
    makeReadOnly(PLANT_LOCATIONS)

    -- local mid = getScreenMiddle()
    -- ANIMATION_BOX = makeBox(mid.x - 60, mid.y - 50, 105, 85)
    -- ARM_BOX = makeBox(mid.x - 90, mid.y - 20, 80, 25)
    BUILD_BTN = Vector:new{31, 135}
end

-- Create a table of direction string -> box. Each box is where we will search the plant placed for that given direction
-- string.
-- Full of janky hardcoded values.
-- TODO: Make debuging this easier, figure out pixel scaling for different resolutions, get rid of magic numbers.
-- function makeSearchBox(direction)
--     local xyWindowSize = srGetWindowSize()
--     local search_size = math.floor(xyWindowSize[0] * SEARCH_BOX_SCALE)
--     local mid = getScreenMiddle()
--     local offset_mid = mid - {math.floor(search_size / 3), math.floor(search_size / 3) }

--     local top_left = offset_mid + direction*40 - Vector:new{20,20 }

--     local box = makeBox(top_left.x,top_left.y, search_size, search_size)
--     box.direction = direction
--     return box
-- end
